==> sett-0.13.5-py2.py3-none-any.whl: sett/libsass.py <==
# -*- coding: utf-8 -*-

import time
import os.path
import collections
import importlib
import traceback
import threading

from sett import optional_import, defaults, ROOT
from sett.utils.dispatch import Dispatcher
from paver.easy import task, info, debug, consume_args, path, error
from paver.deps.six import string_types, moves

observers = optional_import('watchdog.observers')
events = optional_import('watchdog.events')
libsass = optional_import('sass', 'libsass')


# Time to wait for additional events in seconds
DEBOUNCE_WAIT = 0.250


class SassDispatcher(Dispatcher):
    def __init__(self):
        self._sass = Sass.default()

    @Dispatcher.auto
    @Dispatcher.on('watch', -1)
    def compile(self):
        """Launches the compilation of all sass files"""
        debug('Building all files with %s', self._sass)
        self._sass()

    def watch(self):
        """Watches the sass source directories and launches compilation as needed"""
        debug('Building/watching with %s', self._sass)
        try:
            watcher = Watcher(self._sass)
            info('Starting watch %s', self._sass)
            watcher()
        except KeyboardInterrupt:
            pass


@task
@consume_args
def sass(args):
    """
    Usage: sass [compile|watch]

    Shortcut to sassc_watch and sassc_compile.
    """
    sd = SassDispatcher()
    sd(*args)


@task
def sassc_compile():
    """Compile the sass files"""
    sd = SassDispatcher()
    sd.compile()


@task
def sassc_watch():
    """Watch the filesystem for modifications and run sass if needed"""
    sd = SassDispatcher()
    sd.watch()


class EventDispatcher(object):
    """Watchdog compatible event dispatcher. It calls the builder when a file
    is modified and have the right extension"""
    def __init__(self, queue):
        self.extensions = {'.css', '.scss'}
        self.queue = queue

    def dispatch(self, event):
        #  events.FileCreatedEvent triggers FileModifiedEvent
        debug(event)
        if isinstance(event, (events.FileModifiedEvent)) and event.src_path in self:
            self.queue.put(event.src_path)

    def __contains__(self, path):
        stem, ext = os.path.splitext(path)
        if ext not in self.extensions:
            return False
        return True


class Watcher(object):
    """Watchdog compatible event dispatcher. It can be called in the same
    thread or started in another thread.

    >>> with Watcher(Sass.default()):
    ...     call_task('runserver')  # Running an http server and building at the same time

    >>> w = Watcher(Sass.default())
    >>> w()  # Blocks until it's stopped
    """
    def __init__(self, builder):
        self.observer = observers.Observer()
        self.queue = moves.queue.Queue()
        self._thread = None
        self._stop = object()

        self.watches = {}
        for filepath in builder.paths:
            watch = self.observer.schedule(EventDispatcher(self.queue), filepath, recursive=True)
            debug('Watching %s', filepath)
            self.watches[filepath] = watch
        self.builder = builder

    def start(self):
        self.start_observer()
        assert self._thread is None
        self._thread = threading.Thread(target=self.run)
        self._thread.start()

    def stop(self):
        self.stop_observer()
        self.queue.put(self._stop)
        self._thread.join()

    def __call__(self):
        self.start_observer()
        debug('Processing loop')
        try:
            self.run()
        finally:
            self.stop_observer()

    def run(self):
        while True:
            path = self.queue.get()
            if path is self._stop:
                break

            paths = set()
            first_event = time.time()
            try:
                # Debounce: capture all events occuring 250ms after the fist
                delay = DEBOUNCE_WAIT
                while delay > 0:
                    paths.add(self.queue.get(timeout=delay))
                    delay = first_event - time.time() + DEBOUNCE_WAIT

            except moves.queue.Empty:
                pass

            if self._stop in paths:
                break

            for path in paths:
                try:
                    self.builder(path)
                except:
                    traceback.print_exc()

    def start_observer(self):
        debug('Start observer')
        self.observer.start()

    def stop_observer(self):
        debug('Stop observer')
        self.observer.stop()
        self.observer.join()

    def __enter__(self):
        self.start()

    def __exit__(self, exc_value, exc_type, tb):
        self.stop()


class DependencyTracker(object):
    def __init__(self):
        self._deps = set()

    def __contains__(self, file):
        return file in self._deps

    def __call__(self, import_string):
        self._deps.add(import_string)
        return [(import_string, )]


class BaseSass(object):
    @classmethod
    def get_default_paths(self):
        sp = defaults.SASS_PATH
        if not sp:
            return []
        if isinstance(sp, string_types):
            return sp.split(':')
        if isinstance(sp, collections.Iterable):
            return [ROOT.joinpath(d) for d in sp]

        bc = ROOT.joinpath('bower_components/'),
        if bc.isdir():
            return [bc]
        return []

    @classmethod
    def get_default_functions(self):
        functions = []
        functions_names = ([defaults.SASS_FUNCTIONS]
                           if isinstance(defaults.SASS_FUNCTIONS, string_types) else
                           defaults.SASS_FUNCTIONS)

        for sassf in functions_names:
            if isinstance(sassf, string_types):
                module = importlib.import_module(sassf)
                functions.extend(f for f in vars(module).values() if isinstance(f, libsass.SassFunction))
            elif isinstance(sassf, libsass.SassFunction):
                functions.append(sassf)
            else:
                raise TypeError('unexpected %s in defaults.SASS_FUNCTIONS' % sassf)

        return functions

    @classmethod
    def default(cls):
        return cls(
            ROOT.joinpath(defaults.SASS_SRC_DIR, 'scss'),
            ROOT.joinpath(defaults.SASS_BUILD_DIR),
            cls.get_default_paths(),
            cls.get_default_functions(),
        )

    def __init__(self, src, dest, include_paths, functions):
        self._src = src
        self._dest = dest
        self._deps = {}
        self._functions = functions

        paths = [self._src]
        paths.extend(include_paths)
        # Ensure trailing slashes so it does not mess with deps computation
        self._paths = [os.path.join(p, '') for p in paths]

    @property
    def paths(self):
        return self._paths

    def __repr__(self):
        return 'Sass({} -> {}, {})'.format(
            self._src,
            self._dest,
            ':'.join(self._paths),
        )

    def get_compile_kwargs(self):
        return {
            'importers': [],
            'output_style': defaults.SASS_OUTPUT_STYLE,
            'custom_functions': self._functions,
            'include_paths': self._paths,
        }

    def __call__(self, filename=None):
        if filename is None:
            self._build_all()
            return

        filename = path(filename)
        if filename.basename().startswith('_'):
            if not self._deps:
                self._build_all()
                return

            dep_names = {
                os.path.join(filename.dirname()[len(p):], filename.basename().stripext().lstrip('_'))
                for p in self._paths
                if filename.startswith(p)
            }

            if not dep_names:
                return

            for main_file, deps in self._deps.items():
                if any(dep_name in deps for dep_name in dep_names):
                    self._build_file(main_file)
            return

        self._build_file(filename)

    def _build_file(self, filename):
        kwargs = self.get_compile_kwargs()
        dep_tracker = DependencyTracker()
        kwargs['importers'].append((0, dep_tracker))

        infile = self._src.joinpath(filename)
        try:
            result = libsass.compile(filename=infile, **kwargs)
        except Exception as e:
            error('Cannot build %s: %s', infile, e)
            raise

        self._deps[infile] = dep_tracker
        relative_infile = os.path.relpath(infile, self._src)

        outfile = self._dest.joinpath(relative_infile).stripext() + '.css'
        if not outfile.parent.isdir():
            outfile.parent.makedirs_p()

        info('Build %s -> %s', filename, outfile)
        with open(outfile, 'wb') as out_stream:
            out_stream.write(result.encode('utf-8'))

    def _build_all(self):
        for filename in path(self._src).walkfiles():

            if filename.ext in {'.scss', '.sass'} and not filename.basename().startswith('_'):
                self._build_file(filename)


class Sass(BaseSass):
    def get_compile_kwargs(self):
        kw = super(Sass, self).get_compile_kwargs()
        kw.setdefault('importers', []).append((1, Sass.css_import))
        return kw

    @staticmethod
    def css_import(path):
        if not path.startswith('CSS:'):
            return None

        s = path[4:] + '.css'
        return [(s, '',)]

==> sett-0.14.1-py2.py3-none-any.whl: sett/libsass.py <==
# -*- coding: utf-8 -*-

import os.path
import collections
import importlib

from sett import optional_import, defaults, ROOT
from sett.utils.dispatch import Dispatcher
from sett.utils.watcher import Watcher as BaseWatcher
from paver.easy import task, info, debug, consume_args, path, error
from paver.deps.six import string_types


observers = optional_import('watchdog.observers')
events = optional_import('watchdog.events')
libsass = optional_import('sass', 'libsass')


# Time to wait for additional events in seconds
DEBOUNCE_WAIT = 0.250


class Watcher(BaseWatcher):
    def __init__(self, builder):
        assert isinstance(builder, Sass)
        super(Watcher, self).__init__(builder, builder._paths, {'.css', '.scss'})


class SassDispatcher(Dispatcher):
    def __init__(self):
        self._sass = Sass.default()

    @Dispatcher.auto
    @Dispatcher.on('watch', -1)
    def compile(self):
        """Launches the compilation of all sass files"""
        debug('Building all files with %s', self._sass)
        self._sass()

    def watch(self):
        """Watches the sass source directories and launches compilation as needed"""
        debug('Building/watching with %s', self._sass)
        try:
            watcher = BaseWatcher(self._sass, self._sass._paths, {'.css', '.scss'})
            info('Starting watch %s', watcher)
            watcher()
        except KeyboardInterrupt:
            pass


@task
@consume_args
def sass(args):
    """
    Usage: sass [compile|watch]

    Shortcut to sassc_watch and sassc_compile.
    """
    sd = SassDispatcher()
    sd(*args)


@task
def sassc_compile():
    """Compile the sass files"""
    sd = SassDispatcher()
    sd.compile()


@task
def sassc_watch():
    """Watch the filesystem for modifications and run sass if needed"""
    sd = SassDispatcher()
    sd.watch()


class DependencyTracker(object):
    def __init__(self):
        self._deps = set()

    def __contains__(self, file):
        return file in self._deps

    def __call__(self, import_string):
        self._deps.add(import_string)
        return [(import_string, )]


class BaseSass(object):
    @classmethod
    def get_default_paths(self):
        sp = defaults.SASS_PATH
        if not sp:
            return []
        if isinstance(sp, string_types):
            return sp.split(':')
        if isinstance(sp, collections.Iterable):
            return [ROOT.joinpath(d) for d in sp]

        bc = ROOT.joinpath('bower_components/'),
        if bc.isdir():
            return [bc]
        return []

    @classmethod
    def get_default_functions(self):
        functions = []
        functions_names = ([defaults.SASS_FUNCTIONS]
                           if isinstance(defaults.SASS_FUNCTIONS, string_types) else
                           defaults.SASS_FUNCTIONS)

        for sassf in functions_names:
            if isinstance(sassf, string_types):
                module = importlib.import_module(sassf)
                functions.extend(f for f in vars(module).values() if isinstance(f, libsass.SassFunction))
            elif isinstance(sassf, libsass.SassFunction):
                functions.append(sassf)
            else:
                raise TypeError('unexpected %s in defaults.SASS_FUNCTIONS' % sassf)

        return functions

    @classmethod
    def default(cls):
        return cls(
            ROOT.joinpath(defaults.SASS_SRC_DIR, 'scss'),
            ROOT.joinpath(defaults.SASS_BUILD_DIR),
            cls.get_default_paths(),
            cls.get_default_functions(),
        )

    def __init__(self, src, dest, include_paths, functions):
        self._src = src
        self._dest = dest
        self._deps = {}
        self._functions = functions

        paths = [self._src]
        paths.extend(include_paths)
        # Ensure trailing slashes so it does not mess with deps computation
        self._paths = [os.path.join(p, '') for p in paths]

    def __repr__(self):
        return 'Sass({} -> {}, {})'.format(
            self._src,
            self._dest,
            ':'.join(self._paths),
        )

    def get_compile_kwargs(self):
        return {
            'importers': [],
            'output_style': defaults.SASS_OUTPUT_STYLE,
            'custom_functions': self._functions,
            'include_paths': self._paths,
        }

    def __call__(self, filename=None):
        if filename is None:
            self._build_all()
            return
        if isinstance(filename, (list, set, tuple, frozenset)):
            for fname in filename:
                self(fname)
            return

        filename = path(filename)
        if filename.basename().startswith('_'):
            if not self._deps:
                self._build_all()
                return

            dep_names = {
                os.path.join(filename.dirname()[len(p):], filename.basename().stripext().lstrip('_'))
                for p in self._paths
                if filename.startswith(p)
            }

            if not dep_names:
                return

            for main_file, deps in self._deps.items():
                if any(dep_name in deps for dep_name in dep_names):
                    self._build_file(main_file)
            return

        self._build_file(filename)

    def _build_file(self, filename):
        assert isinstance(filename, string_types), '%s is not a string type' % filename

        kwargs = self.get_compile_kwargs()
        dep_tracker = DependencyTracker()
        kwargs['importers'].append((0, dep_tracker))

        infile = self._src.joinpath(filename)
        try:
            result = libsass.compile(filename=infile, **kwargs)
        except Exception as e:
            error('Cannot build %s: %s', infile, e)
            raise

        self._deps[infile] = dep_tracker
        relative_infile = os.path.relpath(infile, self._src)

        outfile = self._dest.joinpath(relative_infile).stripext() + '.css'
        if not outfile.parent.isdir():
            outfile.parent.makedirs_p()

        info('Build %s -> %s', filename, outfile)
        with open(outfile, 'wb') as out_stream:
            out_stream.write(result.encode('utf-8'))

    def _build_all(self):
        for filename in path(self._src).walkfiles():

            if filename.ext in {'.scss', '.sass'} and not filename.basename().startswith('_'):
                self._build_file(filename)


class Sass(BaseSass):
    def get_compile_kwargs(self):
        kw = super(Sass, self).get_compile_kwargs()
        kw.setdefault('importers', []).append((1, Sass.css_import))
        return kw

    @staticmethod
    def css_import(path):
        if not path.startswith('CSS:'):
            return None

        s = path[4:] + '.css'
        return [(s, '',)]
